@@ -2,7 +2,7 @@
 #
 # reportbug_submit module - email and GnuPG functions
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   Copyright (C) 1999-2005 Chris Lawrence
+#   Copyright (C) 1999-2006 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -22,7 +22,7 @@
 #
 # Version ##VERSION##; see changelog for revision history
 #
-# $Id: reportbug_submit.py,v 1.17 2006/06/07 16:57:57 lawrencc Exp $
+# $Id: reportbug_submit.py,v 1.20.2.4 2006/10/11 20:02:14 lawrencc Exp $
 
 import sys
 
@@ -255,7 +255,7 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         message = BetterMIMEText(body, _charset=charset)
 
     # Standard headers
-    message['From'] = rfc2047_encode_address(fromaddr, charset, mua)
+    message['From'] = rfc2047_encode_address(fromaddr, 'utf-8', mua)
     message['To'] = rfc2047_encode_address(sendto, charset, mua)
 
     for (header, value) in headers:
@@ -351,26 +351,34 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         smtp_message = re.sub(r'(?m)^[.]', '..', message)
 
         tryagain = True
+        refused = None
         while tryagain:
             tryagain = False
             ewrite("Connecting to %s via SMTP...\n", smtphost)
             try:
                 conn = smtplib.SMTP(smtphost)
+                response = conn.ehlo()
+                if not (200 <= response[0] <= 299):
+                    conn.helo()
                 if smtptls:
                     conn.starttls()
+                    response = conn.ehlo()
+                    if not (200 <= response[0] <= 299):
+                        conn.helo()
                 if smtpuser:
                     if not smtppasswd:
                         smtppasswd = ui.get_password(
                             'Enter SMTP password for %s@%s: ' %
                             (smtpuser, smtphost))
                     conn.login(smtpuser, smtppasswd)
-                conn.sendmail(fromaddr, toaddrs, smtp_message)
+                refused = conn.sendmail(fromaddr, toaddrs, smtp_message)
                 conn.quit()
             except (socket.error, smtplib.SMTPException), x:
                 # If wrong password, try again...
                 if isinstance(x, smtplib.SMTPAuthenticationError):
-                    ewrite('SMTP error: authentication failed.  Try again.')
+                    ewrite('SMTP error: authentication failed.  Try again.\n')
                     tryagain = True
+                    smtppasswd = None
                     continue
             
                 failed = True
@@ -381,6 +389,16 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                 fh.close()
 
                 ewrite('Wrote bug report to %s\n', msgname)
+        # Handle when some recipients are refused.
+        if refused:
+            for (addr, err) in refused.iteritems():
+                ewrite('Unable to send report to %s: %d %s\n', addr, err[0],
+                       err[1])
+            fh, msgname = TempFile(prefix=tfprefix)
+            fh.write(message)
+            fh.close()
+
+            ewrite('Wrote bug report to %s\n', msgname)
     else:
         try:
             pipe.write(message)
